#include "FollowBall.h"
#include "Events/EventRouter.h"
#include "Events/VisionObjectEvent.h"
#include "Shared/WorldState.h"
#include "Motion/HeadPointerMC.h"
//#include "Motion/WalkMC.h"
#include "Shared/WMclass.h"
#include "Shared/ProjectInterface.h"


const unsigned int PAN_TIMER_ID = 1234;
const float FACING_RIGHT = -80.0;
const float FACING_LEFT = 80.0;


//! Converts degrees to radians
inline double DtoR(double deg) { return (deg/180.0*M_PI); }
inline double RtoD(double rad) { return (rad*180.0/M_PI); }


const int VIS_TIMER_ID	= 1111;

void FollowBall::DoStart() {

	BehaviorBase::DoStart();
	
	cout << "Entering the FollowBall::DoStart() function" << endl;

	//load sounds so that we can play them
	sndman->LoadFile("growl2.wav");

	//prepare motion manager to move head
	headpointer_id = motman->addPersistentMotion(SharedObject<HeadPointerMC>());
	
	leds_mc->cycle(RobotInfo::ModeGreenLEDMask, 1000, 100.0);			//this will cause his back to flash blue
	leds_mc->cycle(RobotInfo::ModeRedLEDMask, 1000, 100.0);
	leds_id = motman->addPersistentMotion(leds_mc);
	
	//listen for events such as visible object recognition
	erouter->addListener(this,EventBase::visObjEGID,ProjectInterface::visOrangeBallSID);

	erouter->addTimer(this,PAN_TIMER_ID,1500,true); //create a new timer, ID=1111, every 500ms, repeat=true
	
	//set the head position to (0,0,0)
	HeadPointerMC * headpointer = (HeadPointerMC*)motman->checkoutMotion(headpointer_id);
	headpointer->setJoints(0,0,0);
	motman->checkinMotion(headpointer_id);
	
	cout << "Leaving FollowBall::DoStart()" << endl;
	
}

void FollowBall::DoStop() {
	cout << "FollowBall::DoStop()" << endl;
	erouter->removeListener(this);
	erouter->removeTimer(this);
	sndman->LoadFile("growl2.wav");
	motman->removeMotion(headpointer_id);
	motman->removeMotion(leds_id);
	BehaviorBase::DoStop();

}
void FollowBall::processEvent(const EventBase& event) 
{
	
	if(event.getSourceID() == ProjectInterface::visOrangeBallSID)
	{
		cout << "Ball seen" << endl;
		//get the location of the ball in AIBO's vision (X,Y) coordinates
		double	horiz =	static_cast<const VisionObjectEvent*>(&event)->getCenterX(); //(rotation) and head pan
		double	vert  =	static_cast<const VisionObjectEvent*>(&event)->getCenterY();		//head tilt
		leds_mc->flash(RobotInfo::FaceLEDMask, 100);			//make the leds flash to indicate that the ball is visible
		CenterHeadOnBall(horiz, vert);
	}
	else if(event.getGeneratorID()==EventBase::timerEGID && event.getSourceID()==PAN_TIMER_ID)
	{
		float pan;
		
		if(isLeft)			//if we are facing left, then turn head to the right
			pan = FACING_RIGHT;
		else				//otherwise turn the head left
			pan = FACING_LEFT;
		
		HeadPointerMC * headpointer= (HeadPointerMC*)motman->checkoutMotion(headpointer_id);
		headpointer->setJoints(0,DtoR(pan),0);			
		motman->checkinMotion(headpointer_id);
		isLeft = !isLeft;

	}
}

void FollowBall::CenterHeadOnBall(float x, float y)
{

	double tilt=state->outputs[HeadOffset+TiltOffset]-y*M_PI/7;
	double pan=state->outputs[HeadOffset+PanOffset]-x*M_PI/6;
	//set a bound for the tilt and pan angles
	if(tilt<DtoR(-50))
		tilt=DtoR(-50);
	if(tilt>DtoR(0))
		tilt=DtoR(0);
	if(pan>DtoR(80))
		pan=DtoR(80);
	if(pan<DtoR(-80))
		pan=DtoR(-80);
	HeadPointerMC * headpointer= (HeadPointerMC*)motman->checkoutMotion(headpointer_id);
	headpointer->setJoints(tilt,pan,0);
	motman->checkinMotion(headpointer_id);

	return;

}
