//-*-c++-*-
#ifndef INCLUDED_FollowBall_h_
#define INCLUDED_FollowBall_h_


#include "Behaviors/BehaviorBase.h"
#include "Motion/MotionManager.h"
#include "Behaviors/Controls/FileBrowserControl.h"
#include "Motion/MotionSequenceMC.h"
#include "Motion/EmergencyStopMC.h"
#include "Motion/LedMC.h"
#include "SoundPlay/SoundManager.h"
#include "Motion/MMAccessor.h"
#include "Shared/TimeET.h"
#include "Shared/Config.h"
#include <string>
#include "DataTypes.h"


//! A simple behavior to chase after any objects seen by the vision system
class FollowBall : public BehaviorBase {
public:
	//!constructor
	FollowBall()
		:	BehaviorBase("FollowBall"), headpointer_id(MotionManager::invalid_MC_ID), 
				leds_id(MotionManager::invalid_MC_ID), isLeft(false)
						
	{
			sndman->PlayFile("growl2.wav");	

	}
	//!destructor
	//virtual ~FollowBall() {}

	//! adds a headpointer and a walker, and a listens for vision events
	virtual void DoStart();

	//! removes motion commands and stops listening
	virtual void DoStop();

	//! sets the head to point at the object and sets the body to move where the head points
	virtual void processEvent(const EventBase& event);

	void CenterHeadOnBall(float x, float y);
			
	static std::string getClassDescription() { return "Follow the ball with the head"; }
	virtual std::string getDescription() const { return getClassDescription(); }

protected:
	
	MotionManager::MC_ID headpointer_id; //!< a HeadPointerMC object
	MotionManager::MC_ID leds_id;		
	SharedObject<LedMC> leds_mc;
	
	bool isLeft;
	
};

/*
*	Author: Jeremiah Savage
*	Last Update: 08/04/2005
*	Description: Basic behavior to pan the head back and forth in search of the ball
*/

#endif

