//-*-c++-*-
#ifndef INCLUDED_KickBall_h_
#define INCLUDED_KickBall_h_

#include "Behaviors/BehaviorBase.h"
#include "Motion/MotionManager.h"
#include "Behaviors/Controls/FileBrowserControl.h"
#include "Motion/MotionSequenceMC.h"
#include "Motion/EmergencyStopMC.h"
#include "Motion/LedMC.h"
#include "SoundPlay/SoundManager.h"
#include "Motion/MMAccessor.h"
#include "Shared/TimeET.h"
#include "Shared/Config.h"
#include <string>
#include "WorldMap.h"
#include "DataTypes.h"
#include "States.h"

const float DESIRED_AREA = 2.0;

//! A simple behavior to chase after any objects seen by the vision system
class KickBall : public BehaviorBase {
public:
	//!constructor
	KickBall()
		:	BehaviorBase("KickBall"), previous_area(0.0), time_lost(0), ballArea(0.0), isChinPressed(false), 
				headpointer_id(MotionManager::invalid_MC_ID), walker_id(MotionManager::invalid_MC_ID), leds_id(MotionManager::invalid_MC_ID), isLeft(false)
						
	{
			sndman->PlayFile("sniff.wav");


			SharedObject<MediumMotionSequenceMC>  p("/ms/data/motion/pounce.pos");
				motman->addPrunableMotion(p);
	

	}
	//!destructor
	//virtual ~KickBall() {}

	//! adds a headpointer and a walker, and a listens for vision events
	virtual void DoStart();

	//! removes motion commands and stops listening
	virtual void DoStop();

	//! sets the head to point at the object and sets the body to move where the head points
	virtual void processEvent(const EventBase& event);

	void CenterHeadOnBall(float x, float y);
	void Walk(float fwd, float side, float rotate);
			
	static std::string getClassDescription() { return "Basic chase, trap, kick program"; }
	virtual std::string getDescription() const { return getClassDescription(); }

protected:
	
	float previous_area; //previous Area
	int time_lost;
	bool isLeft;
	float ballArea;
	bool isChinPressed;
	MotionManager::MC_ID headpointer_id; //!< a HeadPointerMC object
	MotionManager::MC_ID walker_id;      //!< a WalkMC object
	MotionManager::MC_ID leds_id;		//
	//WorldMap * pWorld;					//the worldmap
	SharedObject<LedMC> leds_mc;
	
};

/*
*	Author: Jeremiah Savage
*	Last Update: 08/04/2005
*	Description: Chase + Kick Ball
*/

#endif
