#include "LTUSoccer.h"
#include "Events/EventRouter.h"
#include "Events/VisionObjectEvent.h"
#include "Shared/WorldState.h"
#include "Motion/HeadPointerMC.h"
#include "Motion/WalkMC.h"
#include "Shared/WMclass.h"
#include "Shared/ProjectInterface.h"

//! Converts degrees to radians
inline double DtoR(double deg) { return (deg/180.0*M_PI); }



void LTUSoccer::DoStart() {

	BehaviorBase::DoStart();

	//erouter->addListener(this, EventBase::buttonEGID);

	//INITIALIZE VARS
	//PlayerType... default to Attacker
	
	desired_area = ATTACKER_DESIRED_AREA;
	//previous Area... used to track how close the ball is... default to far away
	previous_area = 999;
	ballArea = 0;
	//subState.. default to paused.
	
	currGState = INITIAL;
	//prior_mode = 999;
	//Timer off
	isTimerOn = false;
	isColorSet = true;
	cout << "Team color set to blue" << endl;
	isPosSet = false;
	isChinPressed = false;
	//timer Count = 0
	time_in_state = 0;
	//motion_waiting = false;

	isPosSet = false;
	

	//load sounds so that we can play them
	sndman->LoadFile("SoccerPr.wav");
	sndman->LoadFile("GoalieOn.wav");
	sndman->LoadFile("ForwardS.wav");
	sndman->LoadFile("ImGonnaS.wav");
	sndman->LoadFile("scooby.wav");
	sndman->LoadFile("3yips.wav");

	//prepare motion manager to move head and walk
	headpointer_id = motman->addPersistentMotion(SharedObject<HeadPointerMC>());
	walker_id = motman->addPersistentMotion(SharedObject<WalkMC>());
	leds_mc->cycle(RobotInfo::ModeGreenLEDMask, 1000, 100.0);			//this will cause his back to flash blue
	leds_mc->cycle(RobotInfo::ModeRedLEDMask, 1000, 100.0);
	leds_id = motman->addPersistentMotion(leds_mc);

	//listen for events from vision (ball) and from buttons
	erouter->addListener(this,EventBase::visObjEGID,ProjectInterface::visOrangeBallSID);
	erouter->addListener(this,EventBase::visObjEGID,ProjectInterface::visYellowGoalSID);
	erouter->addListener(this,EventBase::visObjEGID,ProjectInterface::visBlueBlob);
	erouter->addListener(this,EventBase::visObjEGID,ProjectInterface::visPinkBallSID);
	erouter->addListener(this,EventBase::buttonEGID, HeadFrButOffset);
	erouter->addListener(this,EventBase::buttonEGID, MiddleBackButOffset);
	erouter->addListener(this,EventBase::buttonEGID, RearBackButOffset);
	erouter->addListener(this,EventBase::buttonEGID, FrontBackButOffset);
	erouter->addListener(this,EventBase::buttonEGID, ChinButOffset);
	erouter->addListener(this,EventBase::sensorEGID, ChestIRFrameOffset);
	
	
	erouter->addTimer(this,1000,1000,true);
	erouter->addTimer(this,VIS_TIMER_ID,500,true); //create a new timer, ID=1111, every 500ms, repeat=true
	time_in_state = 0;
	isBtnPressed = false;

	HeadPointerMC * headpointer = (HeadPointerMC*)motman->checkoutMotion(headpointer_id);
	headpointer->setJoints(0,0,0);
	motman->checkinMotion(headpointer_id);
	
}

void LTUSoccer::DoStop() {
	currGState = FINAL;
	erouter->removeListener(this);
	erouter->removeTimer(this);
	motman->removeMotion(headpointer_id);
	motman->removeMotion(walker_id);
	motman->removeMotion(leds_id);
	sndman->ReleaseFile("SoccerPr.wav");
	sndman->ReleaseFile("GoalieOn.wav");
	sndman->ReleaseFile("ImGonnaS.wav");
	sndman->ReleaseFile("ForwardS.wav");
	sndman->ReleaseFile("scooby.wav");
	sndman->ReleaseFile("3yips.wav");
	BehaviorBase::DoStop();
	//delete pWorld;
}

//do something when an event is raised
//this could be cleaned up event-wise (only use a timer when out of view)
//###########################################################
//	Process the event
//###########################################################
void LTUSoccer::processEvent(const EventBase& event) {
	

	//this is where we will process state transitions for the time being
	//there are only a few main states that we will need to worry about: initialize, ready, set, playing, penalized, final
	//state flow      init->ready->set->playing<->penalized->initial
	//final from anywhere


	float x,y,z;
	float distance;
	
	if(event.getGeneratorID()==EventBase::timerEGID)
	{

		if(event.getSourceID()==1000)	//timer 1000 (this is the timer used for the attacker
		{
			time_in_state++;
		}
		else if(event.getSourceID() == VIS_TIMER_ID)		
		{
			
			if(currGState == PLAYING || currGState == SET)
			{
				time_lost++;
				if(time_lost >= 6)
				{
					HeadPointerMC * headpointer= (HeadPointerMC*)motman->checkoutMotion(headpointer_id);
					headpointer->setJoints(0,0,0);					//make AIBO drop his head to signify that he is penalized
					motman->checkinMotion(headpointer_id);

				}
			}
			else
				return;
		}
		

		if(currGState != PLAYING)		//these timer are only used for the playing state
			return;		
		

	}

	if(event.getSourceID()==ProjectInterface::visPinkBallSID || event.getSourceID() == ProjectInterface::visOrangeBallSID)	//prepare the code for the orangeBall
	{
		time_lost = 0;				//reset the lost time since we have found it
		ballArea = static_cast<const VisionObjectEvent*>(&event)->getArea();
		if(currGState != PLAYING && currGState != SET)			//these are the only 2 states where we will want to look for the ball
			return;												//this will reduce a lot of output spam
		
		//make all of the face LEDs light up as well
		leds_mc->flash(RobotInfo::FaceLEDMask, 100);
	}

	//if the event is not a timer event:
	if (INITIAL == currGState)
	{
	
		//check for an event that can move us to the set state
		subState = PLAY_PAUSED;			//aibo's play state
		cout << "The current state is Initialize" << endl;
	
		if(event.getTypeID() == EventBase::activateETID)	//this helps reduce spamming / double function calls
		{
			isBtnPressed = true;
			cout << "Chin button pressed\n";
		}
		
		if(event.getTypeID() == EventBase::deactivateETID)	//check first to see if it is a deactivation event (this is the only thing we process
		{
			switch(event.getSourceID())		//back switchs to mark as goalie/attacker/defender or head button for state transition
				{
					case HeadFrButOffset:				//head button pressed ...move to next state
						
						//head button has been pressed (by induction) and released (by observation)
						//change the state
						cout << "Head button released @ Time: " << time_in_state << endl;
						if( isPosSet && isBtnPressed )			//make sure that we qualify to continue
						{
							currGState = READY;
							time_in_state =0;						//reset counter since moving to new state
							isBtnPressed = false;
							leds_mc->cycle(RobotInfo::ModeBlueLEDMask, 1000, 100.0);
							leds_mc->set(RobotInfo::ModeGreenLEDMask, 0);			//this will make the green ear LEDs stop flashing
							leds_mc->set(RobotInfo::ModeRedLEDMask, 0);
						}
						
						break;
						
					case FrontBackButOffset:						//set the ID to attacker
						if(isPosSet == false){
						player_type = ATTACKER_ID;
						desired_area = ATTACKER_DESIRED_AREA;
						isPosSet = true;
						cout << "Attacker Set!" << endl;
						sndman->PlayFile("ForwardS.wav");
						}
						break;
					case MiddleBackButOffset:						//set the ID to Defender
						if(isPosSet == false) {
						player_type = DEFENDER_ID;
						desired_area = ATTACKER_DESIRED_AREA;
						isPosSet = true;
						cout << "Defender Set!" << endl;
						}
						break;
					case RearBackButOffset:							//set the ID to Goalie
						if(isPosSet == false) {
						player_type = ATTACKER_ID;  //GOALIE_ID;    //Since this does not implement the goalie, force attacker
						desired_area = ATTACKER_DESIRED_AREA;
						isPosSet = true;
						cout << "Goalie Set!" << endl;
						sndman->PlayFile("GoalieOn.wav");
						}
						break;
					case ChinButOffset:
						if(isBtnPressed)				//helps prevent double tapping
						{
							isBtnPressed = false;
							if (tColor == BLUE)
							{
								tColor = RED;
								//change the LED color to RED
								//motman->removeMotion(leds_id);
								//MMAccessor<LedMC> leds_acc(leds_id);
								leds_mc->set(RobotInfo::TlBluLEDMask, 0);
								leds_mc->set(RobotInfo::TlRedLEDMask, 100);
								cout << "Team color is now red\n";
								
							}
							else
							{					
								tColor = BLUE;
								//MMAccessor<LedMC> leds_acc(leds_id);
                                
								leds_mc->set(RobotInfo::TlBluLEDMask, 100);
								leds_mc->set(RobotInfo::TlRedLEDMask, 0);
								cout << "Team Color is now blue\n";
							
							}
						}

			}; //end switch
		}
		return;
				
	} //endif INITIAL == currGState

	else if (READY == currGState)
	{
		
			
		cout << "The current state is READY" << endl;
		//if robot moves to position wait
		//otherwise press the head to move to set state
		if(event.getSourceID() == HeadFrButOffset)
		{
			if(EventBase::activateETID == event.getTypeID())				//this will ensure that a pressed button will not carry between states
				isBtnPressed = true;
			else if(EventBase::deactivateETID == event.getTypeID() && isBtnPressed)
			{
				cout << "Head button released! @ Time: " << time_in_state << endl;
				currGState = SET;
				time_in_state = 0;
				isBtnPressed = false;
				SharedObject<MediumMotionSequenceMC>  p("/ms/data/motion/pounce.pos");
				motman->addPrunableMotion(p);
				leds_mc->set(RobotInfo::ModeBlueLEDMask, 0);
				leds_mc->cycle(RobotInfo::ModeBlueLEDMask, 1000, 100.0);
				leds_mc->cycle(RobotInfo::ModeRedLEDMask, 1000, 100.0);
		
			}
		}

		if(event.getSourceID() == RearBackButOffset)			//used for debugging how the IR sensor works
		{
			//float distance;
			//get the current distance from the IR sensor
			distance = state->sensors[ChestIRDistOffset];
			cout<< "distance: " << distance << endl;
		}
		return;

	}
	else if(SET == currGState )
	{
		
		cout << "The current state is SET" << endl;
		
		//an example of this state is when the dog is sitting, waiting for kickoff

		//check for events (i.e. button press) that will move us into the playing state
		if( event.getSourceID() == HeadFrButOffset)
		{
			if(EventBase::activateETID == event.getTypeID())
			{
				isBtnPressed = true;
			}
			else if(event.getTypeID() == EventBase::deactivateETID && isBtnPressed)
			{
				time_in_state = 0;			//start the timer over to measure how long btn pressed
				isBtnPressed = false;		
				//move to state playing (end of penalty)
				cout << "Head button pressed! @ Time: " << time_in_state << endl;
				currGState = PLAYING;
				subState = PLAY_PLAY;					//playing  set to play
				leds_mc->set(RobotInfo::ModeBlueLEDMask, 0);
				leds_mc->set(RobotInfo::ModeRedLEDMask, 0);
				leds_mc->cycle(RobotInfo::ModeGreenLEDMask, 1000, 100.0); 
			}
		}

	
		else if (event.getSourceID()==ProjectInterface::visPinkBallSID || event.getSourceID()==ProjectInterface::visOrangeBallSID)
		{	
			//ball is seen.  Print it's area
			cout << "Ball Area" << ballArea << endl;

			time_lost = 0;
			double	vert  =	static_cast<const VisionObjectEvent*>(&event)->getCenterY();
			double	horiz =	static_cast<const VisionObjectEvent*>(&event)->getCenterX();
			
			CenterHeadOnBall(horiz, vert);
		

		}
		else if(event.getGeneratorID()==EventBase::timerEGID && event.getSourceID() == VIS_TIMER_ID)		
		{
			if (time_lost >=6)
			{
				//call function to pan AIBO's head
			}
		}

		return;
	}
	else if (PLAYING == currGState)
	{
		double headPan_offset = state->outputs[HeadOffset+PanOffset];
		double	horiz =	static_cast<const VisionObjectEvent*>(&event)->getCenterX(); //(rotation) and head pan
		double	vert  =	static_cast<const VisionObjectEvent*>(&event)->getCenterY();		//head tilt
					
		//cout << "The current state is PLAYING" << endl;
		if( event.getSourceID() == HeadFrButOffset && false)
		{
			if(EventBase::deactivateETID == event.getTypeID())			//use the head press to display the visible area of the ball
			{
				//sndman->PlayFile("3yips.wav");
				//display the last seen area of the ball
				cout << "Ball Area" << ballArea << endl;
				return;
			}
		}
		//first, check to see if any of the back buttons are pressed to mark a penalty
		if( MiddleBackButOffset == event.getSourceID() || RearBackButOffset == event.getSourceID() || FrontBackButOffset ==  event.getSourceID() )
		{
		
			cout << "One of the back buttons was pressed and/or released @ Time: " << time_in_state << endl;
			
			//head less than 3 sec to move back to playing...more than 3 = initial state
			cout << "Head button pressed!" << endl;
			currGState = PENALIZED;
			subState = PLAY_PAUSED;
			leds_mc->set(RobotInfo::ModeGreenLEDMask, 0);
			leds_mc->cycle(RobotInfo::ModeRedLEDMask, 1000, 100.0); 

			//make sure that he stops moving
			WalkMC * walker = (WalkMC*)motman->checkoutMotion(walker_id);
			walker->setTargetVelocity(0,0,0);		
			motman->checkinMotion(walker_id);
			HeadPointerMC * headpointer= (HeadPointerMC*)motman->checkoutMotion(headpointer_id);
			headpointer->setJoints(DtoR(-60),0,DtoR(35));					//make AIBO drop his head to signify that he is penalized
			motman->checkinMotion(headpointer_id);
			cout<<"Stop Walk, Head Up"<<endl;
			time_in_state =0;
			return;
		}

		if( player_type == GOALIE_ID)
        {
			//This portion of the code was used for RoboCup, but has been removed because its author, Steven Kryskala,
			//will offer an improvement on the original source code in a seperate application. 
		
		}


		//if none of the sensors were detected, then we must be playing:
		//determine which playing sub-state we are in and process accordingly
		//cout << "The current Sub State is: ";

		switch (subState)
		{
			case PLAY_PAUSED:
				cout << "PLAY_PAUSED" << endl;
				subState = PLAY_LOOK_FOR_BALL;
				break;
			case PLAY_PLAY:
				cout << "PLAY_PLAY" << endl;
				PrevAreaCounter =0;
				subState = PLAY_LOOK_FOR_BALL;
				time_in_state = 0;
				//this is an obsolete state
				break;
			case PLAY_LOOK_FOR_BALL:
				cout << "PLAY_LOOK_FOR_BALL" << endl;
				
				//call the pan head function

				if((event.getSourceID()==ProjectInterface::visPinkBallSID || event.getSourceID()==ProjectInterface::visOrangeBallSID) 
					&& event.getTypeID()==EventBase::statusETID) 
				{
					subState = PLAY_GO_TO_BALL;
				}

			case PLAY_GO_TO_BALL:
				//call walking function
				//we see the ball, so lets go to it
				//stop when near ball (see distance) then set new state/mode

				//cout << "PLAY_GO_TO_BALL" << endl;
                			

				if(ballArea >= desired_area && abs(headPan_offset + horiz) < DtoR(35) )			//check to see if the visible area of the ball is large (we are close to it)
				{
					subState = PLAY_NEAR_BALL;
					MMAccessor<WalkMC>(walker_id)->setTargetVelocity(0,0,0);
					HeadPointerMC * headpointer= (HeadPointerMC*)motman->checkoutMotion(headpointer_id);
					//						tilt, pan, roll/nod
					headpointer->setJoints(DtoR(-70),0,DtoR(35));					//make AIBO drop his head to try and trap the ball
					motman->checkinMotion(headpointer_id);
					time_in_state = 0;
					return;

				}
			
				if((event.getSourceID()==ProjectInterface::visPinkBallSID || event.getSourceID()==ProjectInterface::visOrangeBallSID) && event.getTypeID()==EventBase::statusETID) 
				{
					/*
					max values for setTargetVelocity:
					x: 180 mm/sec		
					y: 140	mm/sec
					z: 1.8 rad/sec   ~ 103.13 d/sec
					*/
					
					x=180.0f*cos(headPan_offset);			//forward movement
					y=0.0;//120.0f*sin(headPan_offset);			//left/right movement (straife)   --> we don't want this for the time being
					
					z = 1.0f * sin(horiz + headPan_offset);						//adjust the rotation based on the offset
				
					MMAccessor<WalkMC>(walker_id)->setTargetVelocity(x,y,z);

					CenterHeadOnBall(horiz, vert);
				}

				
				break;
			case PLAY_NEAR_BALL:

				if(event.getSourceID() == ChinButOffset && event.getTypeID() == EventBase::activateETID)
				{
					isChinPressed = true;
					cout << "Chin button pressed" << endl;
				}

				if(time_in_state >= 2)				//hold the ball for 2 seconds  before kicking
				{
					if(isChinPressed)				//if the ball is found under the chin
					{
						//try and trap the ball
						HeadPointerMC * headpointer= (HeadPointerMC*)motman->checkoutMotion(headpointer_id);
						headpointer->setJoints(0,0,0);					//return head to normal positioning
						motman->checkinMotion(headpointer_id);
						motman->addPrunableMotion((SharedObject<MediumMotionSequenceMC>)("/ms/data/motion/k_bump.mot"));
					}
						isChinPressed = false;
						subState = PLAY_LOOK_FOR_BALL;
				}
				

				break;
			case PLAY_FIND_GOAL:
				cout << "PLAY_FIND_GOAL" << endl;
				//look for the opposing goal (event only fired when ball is seen)
				/*
				Not yet implemented
				*/
				break;
			case PLAY_GET_INLINE:
				cout << "PLAY_GET_INLINE" << endl;
				//not yet implemented
				break;

		};
				

		
	}

	else if (PENALIZED == currGState)
	{
		subState = PLAY_PAUSED;
		if (time_in_state %  5 == 0)    //temp fix to stop spam
		{
			cout << "The current state is PENALIZED" << endl;
		}

		if(event.getSourceID() == HeadFrButOffset)
		{
			cout << "Head button pressed!" << endl;

			if(event.getTypeID() == EventBase::activateETID)  //btn pressed
			{
				cout << "Button Pressed down" << endl;
				time_in_state = 0;			//start the timer over to measure how long btn pressed
				isBtnPressed = true;		//a slight hack to reuse the counter here
			}
			else if(event.getTypeID() == EventBase::deactivateETID && isBtnPressed == true)		//btn released
			{
				isBtnPressed = false;
				leds_mc->set(RobotInfo::ModeRedLEDMask, 0);
				if (time_in_state < 3)
				{
					//move to state playing (end of penalty)
					cout << "Head button pressed! @ Time: " << time_in_state << endl;
					currGState = PLAYING;
					subState = PLAY_LOOK_FOR_BALL;					//playing  set to play
					
					leds_mc->cycle(RobotInfo::ModeGreenLEDMask, 1000, 100.0); 
			
			
				}
				else
				{
					//move back to the initialize/ready state
					currGState = INITIAL;
					leds_mc->cycle(RobotInfo::ModeGreenLEDMask, 1000, 100.0);			//together these will make the ears blink Yellow
					leds_mc->cycle(RobotInfo::ModeRedLEDMask, 1000, 100.0);				//########################
					
				}
				HeadPointerMC * headpointer = (HeadPointerMC*)motman->checkoutMotion(headpointer_id);	//return head to normal view
				headpointer->setJoints(0,0,0);
				motman->checkinMotion(headpointer_id);


			}
			
			
			return;
		}

	}
	else
	{
		cout << "Unknown state.  Error!!" << endl;
	}

}

void LTUSoccer::CenterHeadOnBall(float x, float y)
{

	double tilt=state->outputs[HeadOffset+TiltOffset]-y*M_PI/7;
	double pan=state->outputs[HeadOffset+PanOffset]-x*M_PI/6;
	//set a bound for the tilt and pan angles
	if(tilt<DtoR(-50))
		tilt=DtoR(-50);
	if(tilt>DtoR(0))
		tilt=DtoR(0);
	if(pan>DtoR(80))
		pan=DtoR(80);
	if(pan<DtoR(-80))
		pan=DtoR(-80);
	HeadPointerMC * headpointer= (HeadPointerMC*)motman->checkoutMotion(headpointer_id);
	headpointer->setJoints(tilt,pan,0);
	motman->checkinMotion(headpointer_id);

	return;


	/*		AIBO JOINT LIMITS
		Parts Range (Unit: degree)
		Neck tilt1 (-75, 0)
		Neck pan (-88, 88)
		Neck tilt2 (-15, 45)
		Mouth (-55, -3)
		Left leg(front/rear)J1 (-115, 130)
		Left leg(front/rear)J2 (-10, 88)
		Left leg(front/rear)J3 (-25, 122)
		Right leg(front/rear)J1 (-130, 115)
		Right leg(front/rear)J2 (-10, 88)
		Right leg(front/rear)J3 (-25, 122)
		Tail tilt (5, 60)
		Tail pan (-45, 45)
	*/
}
